<?php 
/***
 * class PHP odt2xhtml
 * Copyright (C) 2011  Stephane HUC
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact information:
 *   Stephane HUC
 *   <devs@stephane-huc.net>
 *
 ***/

/******************************************************************************/
/***   Dont touch at the rest !                                             ***/
/******************************************************************************/
define('CLASS_ODT2XHTML_ROOT', dirname(__FILE__));

define('ODT2XHTML_MAX_TIME', ini_get('max_execution_time') );
define('ODT2XHTML_MEM', ini_get('memory_limit') );
define('ODT2XHTML_MEM_INT', intval( ODT2XHTML_MEM ) ); 

define('ODT2XHTML_VERSION', 20110310);	// version d'ODT2XHTML

define('ODT2XHTML_XSL_ROOT', ODT2XHTML_ROOT.'/class/xsl');

class odt2xhtml {
	
	public $dir=array();
	public $file=array();
	
	private $attribute='';
	private $body='';
	private $code='';
	private $content='';
	private $debug='';
	private $ext=array();
	private $id='';
	private $html=array();
	private $meta='';
	private $mssg=array();
	private $path='';
	private $pattern='';
	private $position=array();
	private $style='';
	private $title='';
	private $php='';
	private $phpversion=''; /*** php version ***/
	
	public function __construct() { 	// ($root='', $frontend='', $file='')
		
		$this->get_class_need();
		$this->check_constants();
		$this->set_constants();
		$this->set_variables();
		
	}
	
	public function __destruct() {
		
		$this->delete_tmp();
		
	}
	
	/*** Converter ODT TO XHTML ***/
	public function convert2xhtml() {
		
		/*** verify if file's extension is really odf format ****/
		$this->valid_ext();
		
		/*** making all directories ***/
		$this->mk_dirs_needed();
		
		/*** unzip odf file ***/
		$this->unzip_odf();
		
		/***  make new file xml with ODF files xml ***/
		$this->make_new_xml();
		
		/*** move file img to directory img ***/
		$this->mv_img();

		/*** Create temporary file html by xslt processor ***/
		$this->xslt_convert_odf();
		
		/*** Create real files html, css ***/
		$this->create_files();	

		/*** Move icone ***/
		$this->mv_icon();	
		
	}
	
	/*** delete directory temporary ***/
	public function delete_tmp($path='') {	
		try {
			
			if(ODT2XHTML_MEM_INT <= 24) ini_set('memory_limit', '24M');
			ini_set('max_execution_time', 0);
			
			if(empty($path)) $path = $this->dir['odf']['out']['tmp'];
		
			chmod($path, 0777);
			$dir = opendir($path);
		
			/*** tant que la condition est juste en type alors on lit ***/
			while(false !== ($file = readdir($dir))) {
				if($file != '.' && $file != '..') {
					$path_tmp = $path.'/'.$file;
				
					if(!empty($path_tmp) && is_file($path_tmp)) {
						unlink($path_tmp);
					}
					else {
						$this->delete_tmp($path_tmp);
					}
				}
			}
		
			closedir($dir);
			rmdir($path);
		
			if(ODT2XHTML_DEBUG == true) { 
				echo $this->message->display('ok', 'dir_deleted', $path);
			}
		
			if(ini_get('memory_limit') != ODT2XHTML_MEM) ini_set('memory_limit', ODT2XHTML_MEM);
			ini_set('max_execution_time', ODT2XHTML_MAX_TIME);
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** try to obtain elements when file html is created ***/
	public function get_containers_html() {
		try {
			
			$this->file_get_contents($this->dir['odf']['out']['html'].'/'.$this->file['html']);
			
			$this->pattern['meta'] = '`<meta (.*?) />`es';
			
			if(ODT2XHTML_FILE_CSS == false) {
				$this->pattern['css'] = '`<style type="text/css">(.*?)</style>`es';
			}
			
			$this->pattern['body'] = '`<body>(.*?)</body>`es';
			$this->pattern['title'] = '`<h1 class="(.*?)">(.*?)</h1>`es';
		
			foreach($this->pattern as $key => $value) {
				
				preg_match_all($value, $this->content, $this->match); 
				$this->containers[$key] = $this->match;
				
			}
			unset($key, $value);
		
			if(ODT2XHTML_FILE_CSS == true) {
				
				$this->file_get_contents($this->dir['html'].'/'.$this->file['css']);
				
				$this->containers['css'][0][0] = '<style type="text/css">'.$this->content.'</style>';
				$this->containers['css'][1][0] = $this->content;
				
			}
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	public function display_container_html($container, $int) {
		try {
		
			$this->code = '';

			if(is_int($int)) {
				 if($int < 0 || $int > 2) die($this->message->display('ko','container_bad_digit', $int));
				 
				 if($int == 2 && $container != 'title') die($this->message->display('ko', 'container_title', $container));
			}
			else die($this->message->display('ko','bad_digit', $int));
		
			if(!empty($this->containers[$container]) && is_array($this->containers[$container])) {
				foreach($this->containers[$container][$int] as $value) {
					$this->code .= $value."\n";
				}
				unset($value);
				
				if(!empty($this->code)) echo $this->code;
			}
			else die($this->message->display('ko', 'container', $container));
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/***
	    privates functions
	***/
	
	/*** transform values cm in px ***/
	private function analyze_attribute($attribute) {
		try {
			/*
			if(ereg('cm', $attribute)) {
				if(ereg(' ',$attribute)) $xploz = explode(' ', $attribute);
				* */
			if(strpos($attribute, 'cm') !== false) {
				if(strpos($attribute, ' ') !== false) $x = explode(' ', $attribute);
			
				if(!empty($x) && is_array($x)) {
					foreach($x as $k => $v) {
						
						//if(ereg('cm$', $v)) {
						if(preg_match('|cm$|', $v)) {
							$v = round(floatval($v) * 28.6264);
							$x[$k] = $v.'px';
						}

						for($i=0; $i<count($x); $i++) {
							if($i==0) $this->attribute = $x[$i];
							else $this->attribute .= ' '.$x[$i];
						}
						
					}
					unset($k, $v);
				}
				else {
					$this->attribute = round(floatval($attribute) * 28.6264);
					$this->attribute = $this->attribute.'px';
				}
				unset($x);
			}
			else $this->attribute = $attribute;

			$this->attribute = '="'.$this->attribute.'"';
		
			return $this->attribute;
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** choose better file xsl segun file's extension ***/
	function choose_valid_xsl() {
		try {
			
			switch($this->file['ext']) {
				
				case 'odt' :
				case 'ott' :
					$this->file['xsl'] = ODT2XHTML_XSL_ROOT.'/odt2xhtml.xsl';
				break;
				
				case 'sxw' :
				case 'stw' :
					$this->file['xsl'] = ODT2XHTML_XSL_ROOT.'/sxw2xhtml.xsl';
				break;
				
			}
		
			if(ODT2XHTML_DEBUG == true) { echo $this->message->display('ok', 'valid_xsl'); }
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** create file css ***/
	function create_file_css() { 
		try {
			
			$pattern = '/<style type="text\/css">(.*)<\/style>/es';
			if(preg_match_all($pattern, $this->file['tmp'], $this->match))	{
				$this->buffer = trim($this->match[1][0]); 
				
				$this->write_file($this->dir['odf']['out']['html'].'/'.$this->file['css'], 'w', $this->buffer);
				
				unset($this->buffer);
			}
			unset($pattern);
			
			$this->file['tmp'] = $this->replace_content('link_css');

			if(ODT2XHTML_DEBUG == true && !empty($this->file['tmp']) ) { 
				echo $this->message->display('ok','creating_file_css', $this->file['css']); 
			}
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** create ultimate files ***/
	function create_files() {
		try {
		
			/*** modify title in html flux ***/
			$this->file['tmp'] = $this->replace_content('title');
		
			/*** manage to create file css ***/
			if(ODT2XHTML_FILE_CSS == true) $this->create_file_css();

			/*** Create real file html ***/
			$this->write_file($this->dir['odf']['out']['html'].'/'.$this->file['html'], 'w', $this->file['tmp']);
		
			if(ODT2XHTML_DEBUG == true) { 
				echo $this->message->display('ok', 'creating_file_html', $this->file['html']); 
			}
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	protected function build_xml($action) {
		try {
			
			$this->mode = 'ab';
			
			if($action != 'closer' || $action != 'opener') {
				//$this->read_file($this->file['xml']); 
				$this->file_get_contents($this->file['xml']); 
				$this->content = str_replace($this->xml_version, '', $this->content); 
			}
			
			if($action == 'meta' || $action == 'styles') {
				switch($this->file['ext']) {
					
					case 'sxw' :
					case 'stw' :
						$this->content = str_replace($this->doctype, '', $this->content); 
					break;
					
				}
			}
						
			switch($action) {
				
				case 'content':
					/*** try to recuperate text:h level 1 to include in element html title : *** EXPERIMENTAL *** ***/
					$pattern = '/<text:h text:style-name="(.*)" text:outline-level="1">(.*?)<\/text:h>/';
					if(preg_match_all($pattern, $this->content, $this->match)) {
						$this->html['title'] = strip_tags($this->match[2][0]);
					}
			
					/*** add header and footer page ***/
					switch($this->file['ext']) {
					
						case 'odt' :
						case 'ott' :
							# add header page
							if(!empty($this->header)) $this->content = $this->replace_content('header');
							# modify src img
							$this->content = $this->replace_content('img_odt');
							# add footer page
							if(!empty($this->footer)) {
								$search = '</office:text>';
								$replace = $this->footer.'</office:text>';
								$this->content = str_replace($search, $replace, $this->content); 
								unset($search, $replace);
							}
						break;
					
						case 'sxw' :
						case 'stw' :
							$this->content = str_replace($this->doctype, '', $this->content);
							# add header page
							if(!empty($this->header)) $this->content = $this->replace_content('header');
							# modify src img
							$this->content = $this->replace_content('img_sxw');
							# add footer page
							if(!empty($this->footer)) {
								$search = '</office:body>';
								$replace = $this->footer.'</office:body>';
								$this->content = str_replace($search, $replace ,$this->content); 
								unset($search, $replace);
							}
						break;
					
					}
			
					# rebuild text:reference-mark-* in text:reference-mark syntax xml correct : manage element html abbr 
					$this->content = $this->replace_content('reference_mark');

					# analyze attribute to transform style's value cm in px
					$this->content = $this->replace_content('analyze_attribute');
			
					# search text in position indice or exposant to transform it correctly
					$this->rewrite_position();
				break;
				
				case 'closer':
					$this->content = "\n".'</office:document>'."\n";
				break;
				
				case 'styles':
					# analyze attribute to transform style's value cm in px
					$this->content = $this->replace_content('analyze_attribute');
			
					if(preg_match_all('/<style:header>(.*)<\/style:header>/Us', $this->content, $this->match)) {
						$this->header = str_replace('style:header','text:header', $this->match[0][0]); 
					}
				
					if(preg_match_all('/<style:footer>(.*)<\/style:footer>/Us', $this->content, $this->match)) {
						$this->footer = str_replace('style:footer','text:footer', $this->match[0][0]);
					}
				break;
				
				case 'opener':
					$this->content = $this->xml_version."\n";
		
					switch($this->file['ext']) {
				
						case 'odt' :
						case 'ott' :
							$this->content .= $this->replace_content('open_element_xml4odt');
						break;
				
						case 'sxw' :
						case 'stt' :
							$this->content .= $this->replace_content('open_element_xml4sxw');
						break;
				
					}
		
					$this->mode = 'wb';
				break;
				
			}
			
			if(is_array($this->content) && is_string($this->content[1])) {
				$this->write_file($this->file['tmp']['xml'], $this->mode, $this->content[1]); 
			}
			else $this->write_file($this->file['tmp']['xml'], $this->mode, $this->content); 
			
			if(ODT2XHTML_DEBUG == true) { 
				if(is_array($this->content)) echo $this->message->display('ok', 'making_'.$action, serialize($this->content));
				else echo $this->message->display('ok', 'making_'.$action, $this->content); 
			}
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** make image code html ***/
	private function make_image($name) {
		try {
			
			$this->code = 'xlink:href="'.$this->dir['img']['src'].'/php'.$this->php.'/'.$this->file['ext'].'/'.$this->file['name'].'/'.$name.'"';			
			return $this->code;
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** make new file xml with ODT2XHTML files xml ***/
	private function make_new_xml() {
		try {
			
			$this->doctype = '<!DOCTYPE office:document-meta PUBLIC "-//OpenOffice.org//DTD OfficeDocument 1.0//EN" "office.dtd">';
			
			$this->xml_version = '<?xml version="1.0" encoding="UTF-8"?>';
			
			/*** open file xml ***/
			$this->build_xml('opener');
			
			/*** build corpus xml: meta, styles, content ***/
			if(!empty($this->corpus) && is_array($this->corpus)) {
				
				foreach($this->corpus as $value) {
					
					$this->file['xml'] =  $this->file['dir']['tmp'].'/'.$value.'.xml';
					
					/*** modify the content ***/
					$this->build_xml($value); 
					
				}
				unset($value);
				
			}
			
			// close file xml
			$this->build_xml('closer');
			
			if(ODT2XHTML_DEBUG == true) { echo $this->message->display('ok', 'file_xml'); }
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	//private function mk_all_dir() {
	private function mk_dirs_needed() {
		try {
		
			/*** making temporary dir ***/
			$this->mk_dir_tmp();
			
			/*** making dir to receive file html ***/
			$this->mk_dir_html();
			
			/*** making directory to receive file img ***/
			$this->mk_dir_img();
					
			if(ODT2XHTML_DEBUG == true) { echo $this->message->display('ok','making_all_dir'); }
		
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	private function mk_dir_need($dir, $info) {
		try {
			
			//umask(0000);
			// Use DirectoryIterator :p!
			if(!file_exists($dir) and !is_dir($dir)) {
				if(!mkdir($dir, 0777, true)) {
					die($this->message->display('ko','making_dir_'.$info, $dir));
				}	
			}
			else {
				chmod($dir, 0775);
				
				if(ODT2XHTML_DEBUG == true) { 
					echo $this->message->display('ok','making_dir_'.$info, $dir); 
				}
			}
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	private function mk_dir_img() {
		try {
			
			/*** making dir img path ***/
			$this->mk_dir_need($this->dir['img']['path'], 'img_path');
			
			/*** making dir img path php ***/
			$this->mk_dir_need($this->dir['img']['php']['path'], 'img_path_php');
			
			/*** making dir img path php ext ***/
			$this->mk_dir_need($this->dir['img']['php']['ext'], 'img_path_php_ext');
			
			/*** making dir img path php ext ooo ***/
			$this->mk_dir_need($this->dir['img']['php']['OOo'], 'img_path_php_ext_ooo');
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	private function mk_dir_html() {
		try {
			
			$this->mk_dir_need($this->dir['odf']['out']['html'], 'html');
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	private function mk_dir_tmp() {
		try {
			
			$this->mk_dir_need($this->dir['odf']['out']['tmp'], 'odf_out_tmp');
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** modify appel css ***/
	private function modify_css() {
		try {
		
			$this->link_css = '<link rel="stylesheet" href="'.$this->file['css'].'" type="text/css" media="screen" title="Default" />';
			return $this->link_css;
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** modify title code html ***/
	private function modify_title() {
		try {
			
			$this->title = '<head>'."\n\t";
			$this->title .= '<title>&quot;';
			
			if( (ODT2XHTML_TITLE == 'element_title') && !empty($this->html['title']) ) {
				$this->title .= $this->html['title'];
			}
			else $this->title .= $this->file['name'].'.'.$this->file['ext'];
			
			$this->title .= '&quot;';
			$this->title .= ODT2XHTML_PUB;
			$this->title .= '</title>';
			
			//return $this->title;
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** move icon ***/
	private function mv_icon() {
		try {

			foreach($this->icons as $v) {
				if(!copy(ODT2XHTML_ROOT.'/'.$v, $this->dir['img']['path'].'/'.$v)) {
					echo $this->message->display('ko', 'icon_copy', $v);
				}
				else {
					if(ODT2XHTML_DEBUG == true) {
						echo $this->message->display('ok', 'icon_copy', $v);
					}
				}
			}
			unset($v);
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** make directory image and moving images ***/
	private function mv_img() {		
		try {
			// Use directoryIterator :p
			/*** move img ***/
			if(file_exists($this->dir['odf']['out']['pict']) 
				&& $this->handle = opendir($this->dir['odf']['out']['pict']))
			{			
				while( false !== $this->file['img'] = readdir($this->handle) )
				{
					if(is_file($this->dir['odf']['out']['pict'].'/'.$this->file['img']))
					{					
						/*** move img at temp directory to img directory ***/
						if(rename($this->dir['odf']['out']['pict'].'/'.$this->file['img'], $this->dir['img']['php']['OOo'].'/'.$this->file['img'])) {
							chmod($this->dir['img']['php']['OOo'].'/'.$this->file['img'], 0644);
						
							if(ODT2XHTML_DEBUG == true) { echo $this->message->display('ok', 'moving_img', $this->file['img']); }
						}
						else die($this->message->display('ko','moving_img',$this->file['img']));
					}
				}
				closedir($this->handle);
			}
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	/*** replace content ***/
	private function replace_content($info) {
		try {
			$this->exec = true;
			
			switch($info) {
				
				case 'analyze_attribute' :
					$this->search = '/="(.*?)"/es';
					$this->replace = "odt2xhtml::analyze_attribute('$1')";
					$this->subject = $this->content;
				break;
				
				case 'header' :
					$this->search = '!<office:forms(.*?)/>!';
					$this->replace = '<office:forms$1/>'.$this->header;
					$this->subject = $this->content;
				break;
				
				case 'img_odt' :
					$this->search = '#xlink:href="Pictures/([.a-zA-Z_0-9]*)"#es';
					$this->replace = "odt2xhtml::make_image('$1')";
					$this->subject = $this->content;
				break;
				
				case 'img_sxw' :
					$this->search = '!xlink:href="\#Pictures/([.a-zA-Z_0-9]*)"!es';
					$this->replace = "odt2xhtml::make_image('$1')";
					$this->subject = $this->content;
				break;
				
				case 'link_css' :
					$this->search = '/<style type="text\/css">(.*)<\/style>/es';
					$this->replace = "odt2xhtml::modify_css()";
					$this->subject = $this->file['tmp'];
				break;
				
				case 'open_element_xml4odt' :
					$this->exec = false;
					$this->buffer = '<office:document xmlns:office="urn:oasis:names:tc:opendocument:xmlns:office:1.0">';
				break;
				
				case 'open_element_xml4sxw' :
					$this->exec = false;
					$this->buffer = '<office:document xmlns:office="http://openoffice.org/2000/office">';
				break;
				
				case 'reference_mark' :
					$this->search = '/<text:reference-mark-start text:name="(.*)"\/>(.*)<text:reference-mark-end text:name="(.*)"\/>/SU';
					$this->replace = '<text:reference-mark text:name="$1">$2</text:reference-mark>';
					$this->subject = $this->content;
				break;
				
				case 'title' :
					$this->search = '/<head>/es';
					$this->replace = "odt2xhtml::modify_title()";
					$this->subject = $this->file['tmp'];
				break;
				
			}
		
			if($this->exec == true) $this->buffer = preg_replace($this->search, $this->replace, $this->subject);
			
			if(ODT2XHTML_DEBUG == true) { 
				if(is_array($this->buffer)) echo $this->message->display('ok', 'preg_replace', serialize($this->buffer));
				else echo $this->message->display('ok', 'preg_replace', $this->buffer); 
			}
			
			if(!empty($this->buffer)) return $this->buffer;
		
			unset($this->buffer, $this->exec);
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** search text in position indice or exposant and transform it ***/
	private function rewrite_position() {
		try {
			# search styles text-position
			switch($this->file['ext']) {
				
				case 'odt' :
				case 'ott' :
					$pattern = '`<style:style style:name="T([0-9]+)" style:family="text"><style:text-properties style:text-position="(.*?)"/></style:style>`es';
					if(preg_match_all($pattern, $this->content, $this->match)) {
						$this->make_position($this->match);
					}
				break;
				
				case 'sxw' :
				case 'stw' :
					$pattern = '`<style:style style:name="T([0-9]+)" style:family="text"><style:properties style:text-position="(.*?)"/></style:style>`es';
					if(preg_match_all($pattern, $this->content, $this->match)) {
						$this->make_position($this->match);
					}
				break;
				
			}
			unset($this->match);
		
			# search text relative to style text-position
			$pattern = '`<text:span text:style-name="T([0-9]+)">(.*?)</text:span>`es';
			if(!empty($this->position) && preg_match_all($pattern, $this->content, $this->match)) {
				foreach($this->match[1] as $key => $value) {
					
					if(in_array($value, $this->position['name'])) {
						foreach($this->position['name'] as $key2 => $value2) {
							
							if($value2 == $value) {
								# build search text:span
								$this->position['search'][$key2] = '<text:span text:style-name="T'.$this->position['name'][$key2].'">';
								$this->position['search'][$key2] .= $this->match[2][$key];
								$this->position['search'][$key2] .= '</text:span>';
								
								# build replace text:
								$this->position['replace'][$key2] = '<text:'.$this->position['string'][$key2].' text:style-name="T'.$this->position['name'][$key2].'">';
								$this->position['replace'][$key2] .= $this->match[2][$key];
								$this->position['replace'][$key2] .= '</text:'.$this->position['string'][$key2].'>';
							}
							
						}
						unset($key2, $value2);
					}
					
				}
				unset($key, $value);
			}
			unset($this->match);
		
			# replace search text position par replace text position
			if(!empty($this->position['search']) && is_array($this->position['search'])) {
				foreach($this->position['search'] as $key => $value) {
					$this->content = str_replace($value, $this->position['replace'][$key], $this->content);
				}
				unset($key, $value);
			}
			unset($this->position);
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** Unzip file ODT ***/
	private function unzip_odf() {
		try {
			
			if(class_exists('ZipArchive')) {	// Run with PHP Zip !
				$this->archive = new ZipArchive();

				if(ODT2XHTML_PHPCLI == true) {
					$this->zip = ODT2XHTML_ROOT.'/'.$this->file['name'].'.'.$this->file['ext'];
				}
				else $this->zip = ODT2XHTML_OUT.'/'.$this->file['name'].'.'.$this->file['ext'];

				if($this->archive->open($this->zip) !== TRUE) die($this->message->display('ko','zip_open').$this->zip);
			
				$this->archive->extractTo($this->file['dir']['tmp']) or die($this->message->display('ko','zip_extract').$this->zip); 
				$this->archive->close();
				
				if(ODT2XHTML_DEBUG == true) { echo $this->message->display('ok','unzip'); }
			}
			else { // Run with PCLZIP !
				if(require_once(CLASS_ODT2XHTML_ROOT.'/pclzip/pclzip.lib.php') === false) die($this->message->display('ko','pclzip_require'));
				
				if(class_exists('PclZip')) {
					if(ODT2XHTML_PHPCLI == true) {
						$this->archive = new PclZip(ODT2XHTML_ROOT.'/'.$this->file['name'].'.'.$this->file['ext']);
					}
					else $this->archive = new PclZip(ODT2XHTML_OUT.'/'.$this->file['name'].'.'.$this->file['ext']);
		
					if($this->archive->extract(PCLZIP_OPT_PATH, $this->file['dir']['tmp']) == 0) {
						die($this->message->display('ko','zip_extract').$this->archive->errorInfo(true));
					}
		
					if(ODT2XHTML_DEBUG == true) { echo $this->message->display('ok','unzip'); }
				}
				else die($this->message->displaying('ko','zip_disabled'));
			}
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** Verify if extension is really odt ***/
	private function valid_ext() {
		try {
				
			if(!in_array($this->file['ext'], $this->ext['valid'])) {
				die($this->message->display('ko','extension'));
			}
		
			if(ODT2XHTML_DEBUG == true) { echo $this->message->display('ok','extension'); }
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/***
		Protected functions
	***/
	
	/*** ***/
	protected function check_constants() {
		try {
			
			if(!defined('ODT2XHTML_ROOT')) die($this->message->display('ko','error_define', 'ODT2XHTML_ROOT'));
			if(!defined('ODT2XHTML_FILE_ODF')) die($this->message->display('ko','error_define', 'ODT2XHTML_FILE_ODF'));
			if(!defined('ODT2XHTML_FRONTEND')) die($this->message->display('ko','error_define', 'ODT2XHTML_FRONTEND'));
						
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** ***/
	protected function file_handling($file, $info='', $mode='', $resource='') {
		try {
			
			if(require_once(CLASS_ODT2XHTML_ROOT.'/class.file_handling.php')) {
				
				$this->handling = new file_handling(); 
				if($this->handling->verif_file($file)) {
					
					switch($info) {
						
						case 'fgets':
							$this->content = $this->handling->file_object($file, $info); 
						break;
						
						case 'file_get_contents':
							$this->content = $this->handling->file_info($file, $info);
						break;
						
						case 'vars':
							$this->file_obj = $this->handling->file_object($file); 
							
							if(method_exists($this->file_obj, 'getExtension')) {
								$this->file['ext'] = $this->file_obj->getExtension();	// PHP >= 5.3.2
							}
							else $this->file['ext'] = $this->handling->file_info($file, 'pathinfo', 'ext');
							
							if(method_exists($this->file_obj, 'getBasename')) {
								$this->file['name'] = $this->file_obj->getBasename('.'.$this->file['ext']); 	// PHP >= 5.1.2
							}
							else $this->file['name'] = $this->handling->file_info($file, 'pathinfo', 'name');
							
							$this->file['html'] = $this->file['name'].'.html';
							$this->file['css'] = $this->file['name'].'.css'; 
							
						break;
						
						case 'verif':
							return TRUE;
						break;
						
						case 'write': 
							$this->handling->file_object($file, $info, $mode, $resource);
						break;
						
					}
					
				}
				elseif($info == 'write') { 
					$this->handling->file_info($file, $info, $mode, $resource);
				}
				
			}
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** this function is run by method _rewrite_position() ***/
	protected function make_position($match) {
		try {
			
			if(!empty($match) && is_array($match)) {
				foreach($match[1] as $key => $value) {
					
					$this->position['name'][$key] = $value;
					$this->position['string'][$key] = substr($match[2][$key], 0, 3);
					
				}
				unset($key, $value);
			}
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** read file_get_contents ***/
	protected function file_get_contents($file) {
		try {
			
			$this->file_handling($file, 'file_get_contents');
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** read file xml ***/
	protected function read_file($file) {
		try {
			
			$this->file_handling($file, 'fgets');
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	protected function get_class_need() {
		try {
			
			require_once(CLASS_ODT2XHTML_ROOT.'/class.messaging.php');
			
			$this->message = new messaging();
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	protected function set_constants() {
		try {
			
			if(ODT2XHTML_PHPCLI == true) {
				define('ODT2XHTML_OUT', ODT2XHTML_FRONTEND);	
			}
			else define('ODT2XHTML_OUT', ODT2XHTML_ROOT.ODT2XHTML_FRONTEND);
			
			if(ODT2XHTML_DEBUG == true) echo $this->message->display('ok', 'odf_out', ODT2XHTML_OUT);
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	protected function set_variables() {
		try {
			
			// obtain 1rst string php version
			$this->php = substr(PHP_VERSION, 0, 1);	
			
			// extension valid
			$this->ext['valid'] = array('odt', 'ott', 'stw', 'sxw');
			// icons valid
			$this->icons = array ('favicon.ico','icone.png');
			// xml corpus
			$this->corpus = array('meta', 'styles', 'content');
			
			// set variables files an dir
			$this->build_vars_file_handling();
			
			// set file xsl
			$this->choose_valid_xsl();
			
			if(ODT2XHTML_DEBUG == true) {
				var_dump($this->dir, $this->ext, $this->file, $this->icons, $this->corpus);
			}
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	protected function build_vars_file_handling() {
		try {
			
			// obtain dir and file infos,
			$this->file_handling(ODT2XHTML_FILE_ODF, 'vars');
		
			$this->dir['odf']['out']['html'] = ODT2XHTML_OUT.'odt2xhtml';		// dir html
			$this->dir['odf']['out']['tmp'] = ODT2XHTML_OUT.'tmp';	// tmp dir 
			
			$this->dir['img']['path'] = $this->dir['odf']['out']['html'].'/img';		// dir img
			$this->dir['img']['php']['path'] = $this->dir['img']['path'].'/php'.$this->php;
			$this->dir['img']['php']['ext'] = $this->dir['img']['php']['path'].'/'.$this->file['ext'];
			$this->dir['img']['php']['OOo'] = $this->dir['img']['php']['ext'].'/'.$this->file['name'];
			
			/*** define dir for attribute img src ***/
			if(defined('IMG_SRC')) $this->dir['img']['src'] = IMG_SRC.'img';
			else $this->dir['img']['src'] = 'img';

			$this->file['dir']['tmp'] = $this->dir['odf']['out']['tmp'].'/php'.$this->php.'_'.$this->file['name'].'_'.$this->file['ext'];
			$this->file['tmp']['html'] = $this->file['dir']['tmp'].'_tmp.html';	// file html tmp
			$this->file['tmp']['xml'] = $this->file['dir']['tmp'].'.xml';	// file xml tmp
			
			$this->dir['odf']['out']['pict'] = $this->file['dir']['tmp'].'/Pictures';
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** verif file ***/
	protected function verif_file($file) {
		try {
			
			if($this->file_handling($file, 'verif')) return TRUE;
			else return FALSE;
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** write file ***/
	protected function write_file($filename, $mode, $resource) {
		try {
			
			$this->file_handling($filename, 'write', $mode, $resource); 
		
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
	/*** PHP Convert XML ***/
	protected function xslt_convert_odf() 
	{
		try {
			
			if($this->verif_file($this->file['xsl'])) {
				$dom = new DOMDocument();
				$dom->load($this->file['xsl']);	
		
				$xslt = new XSLTProcessor();
				$xslt->importStylesheet($dom);
				
				if($this->verif_file($this->file['tmp']['xml'])) {
					$dom = new DOMDocument();
					$dom->load($this->file['tmp']['xml']);
					
					$this->file['tmp'] = html_entity_decode($xslt->transformToXML($dom));
					
					if(ODT2XHTML_DEBUG == true && !empty($this->file['tmp']) ) { 
						echo $this->message->display('ok','convert_odf', $this->file['tmp']); 
					}
				}
			} 
			
		}
		catch(Exception $e) { die('Error_method: '.__METHOD__.$e->getMessage()); }
	}
	
}

?>
